@@ -653,12 +653,13 @@ sub compile_type_constraint{
     }
     else{
         $self->{compiled_type_constraint} =  sub{
-            my(@args) = @_;
-            local $_ = $args[0];
-            foreach my $c(@checks){
-                return undef if !$c->(@args);
-            }
-            return 1;
+          my(@args) = @_;
+          for ($args[0]) { # local $_ will cancel tie-ness due to perl's bug
+              foreach my $c(@checks){
+                  return undef if !$c->(@args);
+              }
+          }
+          return 1;
         };
     }
     return;
@@ -712,7 +713,7 @@ sub DESTROY {
                 my $demolish = Mouse::Util::get_code_ref($class, 'DEMOLISH')
                     || next;
 
-                $self->$demolish($Mouse::Util::in_global_destruction);
+                $self->$demolish(Mouse::Util::in_global_destruction());
             }
         };
         $@;
@@ -749,7 +750,7 @@ Mouse::PurePerl - A Mouse guts in pure Perl
 
 =head1 VERSION
 
-This document describes Mouse version 0.98
+This document describes Mouse version 1.00
 
 =head1 SEE ALSO
 
