@@ -1,34 +1,15 @@
 #ifndef MOUSE_H
 #define MOUSE_H
 
-#define PERL_NO_GET_CONTEXT
-#include <EXTERN.h>
-#include <perl.h>
-#include <XSUB.h>
+#define PERL_EUPXS_ALWAYS_EXPORT
 
-#include "ppport.h"
-
-/* Portability stuff */
-
-#ifndef newSVpvs_share
-#define newSVpvs_share(s) Perl_newSVpvn_share(aTHX_ s, sizeof(s)-1, 0U)
-#endif
-
-#ifndef get_cvs
-#define get_cvs(name, flags) get_cv(name, flags)
-#endif
-
-#ifndef GvNAME_get
-#define GvNAME_get GvNAME
-#endif
-#ifndef GvNAMELEN_get
-#define GvNAMELEN_get GvNAMELEN
-#endif
+#include "xshelper.h"
 
 #ifndef mro_get_linear_isa
 #define no_mro_get_linear_isa
 #define mro_get_linear_isa(stash) mouse_mro_get_linear_isa(aTHX_ stash)
 AV* mouse_mro_get_linear_isa(pTHX_ HV* const stash);
+#define mro_method_changed_in(stash) ((void)++PL_sub_generation)
 #endif /* !mro_get_linear_isa */
 
 #ifndef mro_get_pkg_gen
@@ -39,35 +20,27 @@ AV* mouse_mro_get_linear_isa(pTHX_ HV* const stash);
 #endif /* !no_mro_get_linear_isa */
 #endif /* mro_get_package_gen */
 
-#if !defined(__GNUC__)
-#  if (!defined(__cplusplus__) || !defined(__STDC_VERSION__) ||  (__STDC_VERSION__ < 199901L)) && !defined(inline)
-#    define inline /* nothing */
-#  endif
+#ifndef GvCV_set
+#define GvCV_set(gv, cv) (GvCV(gv) = (cv))
 #endif
 
-/* Mouse stuff */
-
-#define newAV_mortal() (AV*)sv_2mortal((SV*)newAV())
-#define newHV_mortal() (HV*)sv_2mortal((SV*)newHV())
-
-#define MOUSE_CALL_BOOT(name) STMT_START {      \
-        EXTERN_C XS(CAT2(boot_, name));         \
-        PUSHMARK(SP);                           \
-        CALL_FPTR(CAT2(boot_, name))(aTHX_ cv); \
-    } STMT_END
-
 extern SV* mouse_package;
-extern SV* mouse_namespace;
 extern SV* mouse_methods;
 extern SV* mouse_name;
+extern SV* mouse_coerce;
 
 void
 mouse_throw_error(SV* const metaobject, SV* const data /* not used */, const char* const fmt, ...)
-#ifdef __attribute__format__
     __attribute__format__(__printf__, 3, 4);
-#else
-    ;
-#endif
+
+/* workaround RT #69939 */
+I32
+mouse_call_sv_safe(pTHX_ SV*, I32);
+
+#define call_sv_safe(sv, flags)     mouse_call_sv_safe(aTHX_ sv, flags)
+#define call_method_safe(m, flags)  mouse_call_sv_safe(aTHX_ newSVpvn_flags(m, strlen(m), SVs_TEMP), flags | G_METHOD)
+#define call_method_safes(m, flags) mouse_call_sv_safe(aTHX_ newSVpvs_flags(m, SVs_TEMP),            flags | G_METHOD)
+
 
 #define is_class_loaded(sv) mouse_is_class_loaded(aTHX_ sv)
 bool mouse_is_class_loaded(pTHX_ SV*);
@@ -100,6 +73,13 @@ GV* mouse_stash_fetch(pTHX_ HV* const stash, const char* const name, I32 const n
 #define stash_fetch(s, n, l, c) mouse_stash_fetch(aTHX_ (s), (n), (l), (c))
 #define stash_fetchs(s, n, c)   mouse_stash_fetch(aTHX_ (s), STR_WITH_LEN(n), (c))
 
+void mouse_install_sub(pTHX_ GV* const gv, SV* const code_ref);
+
+void mouse_must_defined(pTHX_ SV* const value, const char* const name);
+void mouse_must_ref(pTHX_ SV* const value, const char* const name, svtype const t);
+
+#define must_defined(sv, name)   mouse_must_defined(aTHX_ sv, name)
+#define must_ref(sv, name, svt)  mouse_must_ref(aTHX_ sv, name, svt)
 
 #define MOUSEf_DIE_ON_FAIL 0x01
 MAGIC* mouse_mg_find(pTHX_ SV* const sv, const MGVTBL* const vtbl, I32 const flags);
@@ -141,30 +121,40 @@ void mouse_instance_weaken_slot(pTHX_ SV* const instance, SV* const slot);
 #define get_slots(self, key)        get_slot(self, sv_2mortal(newSVpvs_share(key)))
 #define set_slots(self, key, value) set_slot(self, sv_2mortal(newSVpvs_share(key)), value)
 
-/* mouse_simle_accessor.xs */
+/* mouse_simle_accessor.xs for meta object protocols */
 #define INSTALL_SIMPLE_READER(klass, name) \
     INSTALL_SIMPLE_READER_WITH_KEY(klass, name, name)
 #define INSTALL_SIMPLE_READER_WITH_KEY(klass, name, key) \
-    (void)mouse_simple_accessor_generate(aTHX_ "Mouse::Meta::" #klass "::" #name, #key, sizeof(#key)-1, XS_Mouse_simple_reader, NULL, 0)
+    (void)mouse_simple_accessor_generate(aTHX_ "Mouse::Meta::" #klass "::" \
+    #name, #key, sizeof(#key)-1, XS_Mouse_simple_reader, NULL, 0)
 
 #define INSTALL_CLASS_HOLDER_SV(klass, name, dsv) \
-    (void)mouse_simple_accessor_generate(aTHX_ "Mouse::Meta::" #klass "::" #name, #name, sizeof(#name)-1, XS_Mouse_simple_reader, (dsv), HEf_SVKEY)
+    (void)mouse_simple_accessor_generate(aTHX_ "Mouse::Meta::" #klass "::" \
+    #name, #name, sizeof(#name)-1, XS_Mouse_simple_reader, (dsv), HEf_SVKEY)
 #define INSTALL_CLASS_HOLDER(klass, name, ds) \
     INSTALL_CLASS_HOLDER_SV(klass, name, newSVpvs(ds))
 
 #define INSTALL_SIMPLE_WRITER(klass, name) \
     NSTALL_SIMPLE_WRITER_WITH_KEY(klass, name, name)
 #define INSTALL_SIMPLE_WRITER_WITH_KEY(klass, name, key) \
-    (void)mouse_simple_accessor_generate(aTHX_ "Mouse::Meta::" #klass "::" #name, #key, sizeof(#key)-1, XS_Mouse_simple_writer, NULL, 0)
+    (void)mouse_simple_accessor_generate(aTHX_ "Mouse::Meta::" #klass "::" \
+    #name, #key, sizeof(#key)-1, XS_Mouse_simple_writer, NULL, 0)
 
 #define INSTALL_SIMPLE_PREDICATE(klass, name) \
     INSTALL_SIMPLE_PREDICATE_WITH_KEY(klass, name, name)
 #define INSTALL_SIMPLE_PREDICATE_WITH_KEY(klass, name, key) \
-    (void)mouse_simple_accessor_generate(aTHX_ "Mouse::Meta::" #klass "::" #name, #key, sizeof(#key)-1, XS_Mouse_simple_predicate, NULL, 0)
+    (void)mouse_simple_accessor_generate(aTHX_ "Mouse::Meta::" #klass "::" \
+    #name, #key, sizeof(#key)-1, XS_Mouse_simple_predicate, NULL, 0)
+
+/* generate inhertiable class accessors for Mouse::Meta::Class */
+#define INSTALL_INHERITABLE_CLASS_ACCESSOR(name) \
+    INSTALL_INHERITABLE_CLASS_ACCESSOR_WITH_KEY(name, name)
+#define INSTALL_INHERITABLE_CLASS_ACCESSOR_WITH_KEY(name, key) \
+    (void)mouse_simple_accessor_generate(aTHX_ "Mouse::Meta::Class::" #name,\
+    #key, sizeof(#key)-1, XS_Mouse_inheritable_class_accessor, NULL, 0)
 
 CV* mouse_simple_accessor_generate(pTHX_ const char* const fq_name, const char* const key, I32 const keylen, XSUBADDR_t const accessor_impl, void* const dptr, I32 const dlen);
 
-XS(XS_Mouse_simple_accessor);
 XS(XS_Mouse_simple_reader);
 XS(XS_Mouse_simple_writer);
 XS(XS_Mouse_simple_clearer);
@@ -176,6 +166,8 @@ XS(XS_Mouse_accessor);
 XS(XS_Mouse_reader);
 XS(XS_Mouse_writer);
 
+XS(XS_Mouse_inheritable_class_accessor);
+
 /* type constraints */
 
 int mouse_tc_check(pTHX_ SV* const tc, SV* const sv);
@@ -205,8 +197,6 @@ CV* mouse_generate_can_predicate_for(pTHX_ SV* const klass, const char* const pr
 
 int mouse_is_an_instance_of(pTHX_ HV* const stash, SV* const instance);
 
-XS(XS_Mouse_constraint_check);
-
 /* Mouse XS Attribute object */
 
 AV* mouse_get_xa(pTHX_ SV* const attr);
@@ -257,6 +247,7 @@ enum mouse_xa_flags_t{
     MOUSEf_MOUSE_MASK           = 0xFFFF /* not used */
 };
 
-
+/* Mouse::Meta::Class stuff */
+HV* mouse_get_namespace(pTHX_ SV* const meta); /* $meta->namespace */
 #endif /* !MOUSE_H */
 
